@@ -1,4 +1,4 @@
-use crate::ebr::{Arc, AtomicArc, Barrier, Ptr, Tag};
+use super::ebr::{Arc, AtomicArc, Barrier, Ptr, Tag};
 
 use std::sync::atomic::Ordering::{self, Relaxed, Release};
 
@@ -15,8 +15,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::{AtomicArc, Tag};
     /// use scc::LinkedList;
+    /// use scc::ebr::{AtomicArc, Tag};
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -45,8 +45,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::AtomicArc;
     /// use scc::LinkedList;
+    /// use scc::ebr::AtomicArc;
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -72,8 +72,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::AtomicArc;
     /// use scc::LinkedList;
+    /// use scc::ebr::AtomicArc;
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -100,8 +100,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::AtomicArc;
     /// use scc::LinkedList;
+    /// use scc::ebr::AtomicArc;
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -128,8 +128,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use scc::LinkedList;
+    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -159,8 +159,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::AtomicArc;
     /// use scc::LinkedList;
+    /// use scc::ebr::AtomicArc;
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -192,8 +192,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use scc::LinkedList;
+    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -228,7 +228,7 @@ pub trait LinkedList: 'static + Sized {
         while next_ptr.tag() != Tag::Second {
             entry
                 .link_ref()
-                .swap((next_ptr.try_into_arc(), Tag::None), Relaxed);
+                .swap((next_ptr.get_arc(), Tag::None), Relaxed);
             match self
                 .link_ref()
                 .compare_exchange(next_ptr, (Some(entry), new_tag), order, Relaxed)
@@ -254,8 +254,8 @@ pub trait LinkedList: 'static + Sized {
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use scc::LinkedList;
+    /// use scc::ebr::{Arc, AtomicArc, Barrier};
     /// use std::sync::atomic::Ordering::Relaxed;
     ///
     /// #[derive(Default)]
@@ -300,7 +300,7 @@ pub trait LinkedList: 'static + Sized {
         if update_self && self_tag != Tag::Second {
             let _result = self.link_ref().compare_exchange(
                 self_next_ptr,
-                (next_valid_ptr.try_into_arc(), self_tag),
+                (next_valid_ptr.get_arc(), self_tag),
                 Release,
                 Relaxed,
             );
