@@ -71,7 +71,6 @@ mod hashmap_test {
                         scanned += 1;
                         checker.insert(*k);
                     });
-                    println!("scanned: {}, max: {}", scanned, max);
                     for key in 0..max {
                         assert!(checker.contains(&key));
                     }
@@ -85,7 +84,6 @@ mod hashmap_test {
                         scanned += 1;
                         assert!(*k < max);
                     });
-                    println!("scanned: {}, max: {}", scanned, max);
                 }
             });
             // insert
@@ -265,14 +263,11 @@ mod hashindex_test {
                 // test insert
                 for _ in 0..2 {
                     barrier_copied.wait();
-                    let mut scanned = 0;
                     let mut checker = BTreeSet::new();
                     let max = inserted_copied.load(Acquire);
                     for iter in hashindex_copied.iter(&ebr::Barrier::new()) {
-                        scanned += 1;
                         checker.insert(*iter.0);
                     }
-                    println!("scanned: {}, max: {}", scanned, max);
                     for key in 0..max {
                         assert!(checker.contains(&key));
                     }
@@ -280,13 +275,10 @@ mod hashindex_test {
                 // test remove
                 for _ in 0..2 {
                     barrier_copied.wait();
-                    let mut scanned = 0;
                     let max = removed_copied.load(Acquire);
                     for iter in hashindex_copied.iter(&ebr::Barrier::new()) {
-                        scanned += 1;
                         assert!(*iter.0 < max);
                     }
-                    println!("scanned: {}, max: {}", scanned, max);
                 }
             });
             // insert
@@ -314,9 +306,8 @@ mod hashindex_test {
 
 #[cfg(test)]
 mod treeindex_test {
-    use crate::TreeIndex;
-
     use crate::ebr;
+    use crate::TreeIndex;
 
     use proptest::strategy::{Strategy, ValueTree};
     use proptest::test_runner::TestRunner;
@@ -451,7 +442,7 @@ mod treeindex_test {
             }));
         }
         barrier.wait();
-        for i in 0..512 {
+        for _ in 0..512 {
             let mut found_0 = false;
             let mut found_markers = 0;
             let mut prev_marker = 0;
@@ -472,9 +463,6 @@ mod treeindex_test {
                 assert!(prev == 0 || prev < current);
                 prev = current;
             }
-            if i % 256 == 0 {
-                println!("{} {}", i, found_markers);
-            }
             assert!(found_0);
             assert_eq!(found_markers, num_threads);
         }
@@ -633,3 +621,149 @@ mod treeindex_test {
         }
     }
 }
+
+#[cfg(test)]
+mod hashmap_test_async {
+    use crate::awaitable::HashMap;
+
+    use std::sync::atomic::AtomicUsize;
+    use std::sync::atomic::Ordering::Relaxed;
+    use std::sync::Arc;
+
+    use tokio::sync::Barrier;
+
+    struct R(&'static AtomicUsize);
+    impl R {
+        fn new(cnt: &'static AtomicUsize) -> R {
+            cnt.fetch_add(1, Relaxed);
+            R(cnt)
+        }
+    }
+    impl Drop for R {
+        fn drop(&mut self) {
+            self.0.fetch_sub(1, Relaxed);
+        }
+    }
+
+    #[tokio::test]
+    async fn insert_drop() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let hashmap: HashMap<usize, R> = HashMap::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(hashmap.insert(k, R::new(&CNT)).await.is_ok());
+        }
+        assert_eq!(CNT.load(Relaxed), workload_size);
+        assert_eq!(hashmap.len(), workload_size);
+        drop(hashmap);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(crate::ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test]
+    async fn clear() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let hashmap: HashMap<usize, R> = HashMap::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(hashmap.insert(k, R::new(&CNT)).await.is_ok());
+        }
+        assert_eq!(CNT.load(Relaxed), workload_size);
+        assert_eq!(hashmap.len(), workload_size);
+        assert_eq!(hashmap.clear().await, workload_size);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(crate::ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test(flavor = "multi_thread", worker_threads = 16)]
+    async fn integer_key() {
+        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
+
+        let num_tasks = 8;
+        let workload_size = 256;
+        let mut task_handles = Vec::with_capacity(num_tasks);
+        let barrier = Arc::new(Barrier::new(num_tasks));
+        for task_id in 0..num_tasks {
+            let barrier_cloned = barrier.clone();
+            let hashmap_cloned = hashmap.clone();
+            task_handles.push(tokio::task::spawn(async move {
+                barrier_cloned.wait().await;
+                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
+                for id in range.clone() {
+                    let result = hashmap_cloned.insert(id, id).await;
+                    assert!(result.is_ok());
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.read(&id, |_, v| *v).await;
+                    assert_eq!(result, Some(id));
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.remove_if(&id, |v| *v == id).await;
+                    assert_eq!(result, Some((id, id)));
+                }
+                for id in range {
+                    let result = hashmap_cloned.remove_if(&id, |v| *v == id).await;
+                    assert_eq!(result, None);
+                }
+            }));
+        }
+
+        for r in futures::future::join_all(task_handles).await {
+            assert!(r.is_ok());
+        }
+
+        assert_eq!(hashmap.len(), 0);
+    }
+
+    #[tokio::test(flavor = "multi_thread", worker_threads = 4)]
+    async fn retain_for_each() {
+        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
+
+        let num_tasks = 8;
+        let workload_size = 256;
+        let mut task_handles = Vec::with_capacity(num_tasks);
+        let barrier = Arc::new(Barrier::new(num_tasks));
+        for task_id in 0..num_tasks {
+            let barrier_cloned = barrier.clone();
+            let hashmap_cloned = hashmap.clone();
+            task_handles.push(tokio::task::spawn(async move {
+                barrier_cloned.wait().await;
+                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
+                for id in range.clone() {
+                    let result = hashmap_cloned.insert(id, id).await;
+                    assert!(result.is_ok());
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.insert(id, id).await;
+                    assert_eq!(result, Err((id, id)));
+                }
+                let mut iterated = 0;
+                hashmap_cloned
+                    .for_each(|k, _| {
+                        if range.contains(k) {
+                            iterated += 1;
+                        }
+                    })
+                    .await;
+                assert!(iterated >= workload_size);
+
+                let (_, removed) = hashmap_cloned.retain(|k, _| !range.contains(k)).await;
+                assert_eq!(removed, workload_size);
+            }));
+        }
+
+        for r in futures::future::join_all(task_handles).await {
+            assert!(r.is_ok());
+        }
+
+        assert_eq!(hashmap.len(), 0);
+    }
+}
