@@ -5,11 +5,12 @@ mod leaf;
 mod leaf_node;
 mod node;
 
-use crate::ebr::{Arc, AtomicArc, Barrier, Tag};
+use super::ebr::{Arc, AtomicArc, Barrier, Tag};
 
 use error::{InsertError, RemoveError, SearchError};
 use leaf::{Leaf, Scanner};
 use node::Node;
+
 use std::borrow::Borrow;
 use std::cmp::Ordering;
 use std::fmt;
@@ -220,8 +221,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::Barrier;
     /// use scc::TreeIndex;
+    /// use scc::ebr::Barrier;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
@@ -359,8 +360,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::Barrier;
     /// use scc::TreeIndex;
+    /// use scc::ebr::Barrier;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
@@ -386,8 +387,8 @@ where
     /// # Examples
     ///
     /// ```
-    /// use scc::ebr::Barrier;
     /// use scc::TreeIndex;
+    /// use scc::ebr::Barrier;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
